/* This file was generated by syncqt. */
#ifndef QT_ENGINIO_VERSION_H
#define QT_ENGINIO_VERSION_H

#define ENGINIO_VERSION_STR "1.6.2"

#define ENGINIO_VERSION 0x010602

#endif // QT_ENGINIO_VERSION_H
