/* This file was generated by syncqt. */
#ifndef QT_ACTIVEQT_VERSION_H
#define QT_ACTIVEQT_VERSION_H

#define ACTIVEQT_VERSION_STR "5.6.2"

#define ACTIVEQT_VERSION 0x050602

#endif // QT_ACTIVEQT_VERSION_H
